@@ -33,9 +33,30 @@ public:
 
 protected:
   virtual int InitializeInternal();
-  virtual int CompressFiles(const char* outFileName, const char* toplevel,
-    const std::vector<std::string>& files);
+  /**
+   * This method factors out the work done in component packaging case.
+   */
+  int PackageOnePack(std::string initialToplevel, std::string packageName);
+  /**
+   * The method used to package files when component
+   * install is used. This will create one
+   * archive for each component group.
+   */
+  int PackageComponents(bool ignoreGroup);
+  /**
+   * Special case of component install where all
+   * components will be put in a single installer.
+   */
+  int PackageComponentsAllInOne();
+  virtual int PackageFiles();
   virtual const char* GetOutputExtension() { return ".deb"; }
+  virtual bool SupportsComponentInstallation() const;
+  virtual std::string GetComponentInstallDirNameSuffix(
+      const std::string& componentName);
+
+private:
+  int createDeb();
+  std::vector<std::string> packageFiles;
 
 };
 
