@@ -57,7 +57,7 @@ static const char* cmCTestErrorMatches[] = {
   "^ild:([ \\t])*\\(undefined symbol\\)",
   "([^ :]+) : (error|fatal error|catastrophic error)",
   "([^:]+): (Error:|error|undefined reference|multiply defined)",
-  "([^:]+)\\(([^\\)]+)\\) : (error|fatal error|catastrophic error)",
+  "([^:]+)\\(([^\\)]+)\\) ?: (error|fatal error|catastrophic error)",
   "^fatal error C[0-9]+:",
   ": syntax error ",
   "^collect2: ld returned 1 exit status",
@@ -93,6 +93,8 @@ static const char* cmCTestErrorMatches[] = {
   ": No such file or directory",
   ": Invalid argument",
   "^The project cannot be built\\.",
+  "^\\[ERROR\\]",
+  "^Command .* failed with exit code",
   0
 };
 
@@ -119,7 +121,7 @@ static const char* cmCTestWarningMatches[] = {
   "^\"[^\"]+\", line [0-9]+: [Ww](arning|arnung)",
   "([^:]+): warning[ \\t]*[0-9]+[ \\t]*:",
   "^(Warning|Warnung) ([0-9]+):",
-  "^(Warning|Warnung) ",
+  "^(Warning|Warnung)[ :]",
   "WARNING: ",
   "([^ :]+) : warning",
   "([^:]+): warning",
@@ -131,6 +133,7 @@ static const char* cmCTestWarningMatches[] = {
   "\".*\", line [0-9]+: remark\\([0-9]*\\):",
   "cc-[0-9]* CC: REMARK File = .*, Line = [0-9]*",
   "^CMake Warning.*:",
+  "^\\[WARNING\\]",
   0
 };
 
