@@ -12,12 +12,21 @@
 #include "cmEndWhileCommand.h"
 
 bool cmEndWhileCommand
-::InvokeInitialPass(std::vector<cmListFileArgument> const&,
+::InvokeInitialPass(std::vector<cmListFileArgument> const& args,
                     cmExecutionStatus &)
 {
-  this->SetError("An ENDWHILE command was found outside of a proper "
-                 "WHILE ENDWHILE structure. Or its arguments did not "
-                 "match the opening WHILE command.");
+  if (args.empty())
+    {
+    this->SetError("An ENDWHILE command was found outside of a proper "
+                   "WHILE ENDWHILE structure.");
+    }
+  else
+    {
+    this->SetError("An ENDWHILE command was found outside of a proper "
+                   "WHILE ENDWHILE structure. Or its arguments did not "
+                   "match the opening WHILE command.");
+    }
+
   return false;
 }
 
