@@ -9,7 +9,10 @@
   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the License for more information.
 ============================================================================*/
-#include "cmSystemTools.h"   
+#if defined(_MSC_VER) && _MSC_VER < 1300
+# define _WIN32_WINNT 0x0400 /* for wincrypt.h */
+#endif
+#include "cmSystemTools.h"
 #include <ctype.h>
 #include <errno.h>
 #include <time.h>
@@ -31,7 +34,9 @@
 
 #if defined(_WIN32)
 # include <windows.h>
+# include <wincrypt.h>
 #else
+# include <sys/time.h>
 # include <sys/types.h>
 # include <unistd.h>
 # include <utime.h>
@@ -49,7 +54,7 @@
 #if defined(CMAKE_BUILD_WITH_CMAKE)
 #  include <memory> // auto_ptr
 #  include <fcntl.h>
-#  include <cmsys/MD5.h>
+#  include "cmCryptoHash.h"
 #endif
 
 #if defined(CMAKE_USE_ELF_PARSER)
@@ -121,11 +126,13 @@ const char* cmSystemTools::GetWindows9xComspecSubstitute()
   return cmSystemTools::s_Windows9xComspecSubstitute.c_str();
 }
 
-void (*cmSystemTools::s_ErrorCallback)(const char*, const char*, 
+void (*cmSystemTools::s_ErrorCallback)(const char*, const char*,
                                        bool&, void*);
 void (*cmSystemTools::s_StdoutCallback)(const char*, int len, void*);
 void* cmSystemTools::s_ErrorCallbackClientData = 0;
 void* cmSystemTools::s_StdoutCallbackClientData = 0;
+bool (*cmSystemTools::s_InterruptCallback)(void*);
+void* cmSystemTools::s_InterruptCallbackClientData = 0;
 
 // replace replace with with as many times as it shows up in source.
 // write the result into source.
@@ -139,7 +146,7 @@ void cmSystemTools::ExpandRegistryValues(std::string& source, KeyWOW64 view)
   // a close square-bracket.  The ']' character must be the first in the
   // list of characters inside the [^...] block of the expression.
   cmsys::RegularExpression regEntry("\\[(HKEY[^]]*)\\]");
-  
+
   // check for black line or comment
   while (regEntry.find(source))
     {
@@ -191,6 +198,20 @@ std::string cmSystemTools::EscapeQuotes(const char* str)
   return result;
 }
 
+std::string cmSystemTools::TrimWhitespace(const std::string& s)
+{
+  std::string::const_iterator start = s.begin();
+  while(start != s.end() && *start == ' ')
+    ++start;
+  if (start == s.end())
+    return "";
+
+  std::string::const_iterator stop = s.end()-1;
+  while(*stop == ' ')
+    --stop;
+  return std::string(start, stop+1);
+}
+
 void cmSystemTools::Error(const char* m1, const char* m2,
                           const char* m3, const char* m4)
 {
@@ -215,6 +236,20 @@ void cmSystemTools::Error(const char* m1, const char* m2,
   cmSystemTools::Message(message.c_str(),"Error");
 }
 
+void cmSystemTools::SetInterruptCallback(InterruptCallback f, void* clientData)
+{
+  s_InterruptCallback = f;
+  s_InterruptCallbackClientData = clientData;
+}
+
+bool cmSystemTools::GetInterruptFlag()
+{
+  if(s_InterruptCallback)
+    {
+    return (*s_InterruptCallback)(s_InterruptCallbackClientData);
+    }
+  return false;
+}
 
 void cmSystemTools::SetErrorCallback(ErrorCallback f, void* clientData)
 {
@@ -232,7 +267,7 @@ void cmSystemTools::Stdout(const char* s)
 {
   if(s_StdoutCallback)
     {
-    (*s_StdoutCallback)(s, static_cast<int>(strlen(s)), 
+    (*s_StdoutCallback)(s, static_cast<int>(strlen(s)),
                         s_StdoutCallbackClientData);
     }
   else
@@ -242,6 +277,12 @@ void cmSystemTools::Stdout(const char* s)
     }
 }
 
+void cmSystemTools::Stderr(const char* s, int length)
+{
+    std::cerr.write(s, length);
+    std::cerr.flush();
+}
+
 void cmSystemTools::Stdout(const char* s, int length)
 {
   if(s_StdoutCallback)
@@ -263,7 +304,7 @@ void cmSystemTools::Message(const char* m1, const char *title)
     }
   if(s_ErrorCallback)
     {
-    (*s_ErrorCallback)(m1, title, s_DisableMessages, 
+    (*s_ErrorCallback)(m1, title, s_DisableMessages,
                        s_ErrorCallbackClientData);
     return;
     }
@@ -271,7 +312,7 @@ void cmSystemTools::Message(const char* m1, const char *title)
     {
     std::cerr << m1 << std::endl << std::flush;
     }
-  
+
 }
 
 
@@ -306,7 +347,7 @@ bool cmSystemTools::IsOn(const char* val)
     return false;
     }
   std::basic_string<char> v = val;
-  
+
   for(std::basic_string<char>::iterator c = v.begin();
       c != v.end(); c++)
     {
@@ -339,13 +380,13 @@ bool cmSystemTools::IsOff(const char* val)
     return true;
     }
   std::basic_string<char> v = val;
-  
+
   for(std::basic_string<char>::iterator c = v.begin();
       c != v.end(); c++)
     {
     *c = static_cast<char>(toupper(*c));
     }
-  return (v == "OFF" || v == "0" || v == "NO" || v == "FALSE" || 
+  return (v == "OFF" || v == "0" || v == "NO" || v == "FALSE" ||
           v == "N" || cmSystemTools::IsNOTFOUND(v.c_str()) || v == "IGNORE");
 }
 
@@ -440,6 +481,13 @@ public:
       args.push_back(*arg);
       }
     }
+  void Store(std::vector<cmStdString>& args) const
+    {
+    for(char** arg = this->ArgV; arg && *arg; ++arg)
+      {
+      args.push_back(*arg);
+      }
+    }
 };
 
 //----------------------------------------------------------------------------
@@ -451,6 +499,15 @@ void cmSystemTools::ParseUnixCommandLine(const char* command,
   argv.Store(args);
 }
 
+//----------------------------------------------------------------------------
+void cmSystemTools::ParseUnixCommandLine(const char* command,
+                                         std::vector<cmStdString>& args)
+{
+  // Invoke the underlying parser.
+  cmSystemToolsArgV argv = cmsysSystem_Parse_CommandForUnix(command, 0);
+  argv.Store(args);
+}
+
 std::string cmSystemTools::EscapeWindowsShellArgument(const char* arg,
                                                       int shell_flags)
 {
@@ -548,15 +605,15 @@ std::vector<cmStdString> cmSystemTools::ParseArguments(const char* command)
       args.push_back(arg);
       }
     }
-  
+
   return args;
 }
 
 
 bool cmSystemTools::RunSingleCommand(std::vector<cmStdString>const& command,
                                      std::string* output ,
-                                     int* retVal , const char* dir , 
-                                     bool verbose ,
+                                     int* retVal , const char* dir ,
+                                     OutputOption outputflag ,
                                      double timeout )
 {
   std::vector<const char*> argv;
@@ -578,46 +635,62 @@ bool cmSystemTools::RunSingleCommand(std::vector<cmStdString>const& command,
     {
     cmsysProcess_SetOption(cp, cmsysProcess_Option_HideWindow, 1);
     }
+
   cmsysProcess_SetTimeout(cp, timeout);
   cmsysProcess_Execute(cp);
-  
+
   std::vector<char> tempOutput;
   char* data;
   int length;
-  if ( output || verbose )
+  int pipe;
+  if ( output || outputflag != OUTPUT_NONE )
     {
-  while(cmsysProcess_WaitForData(cp, &data, &length, 0))
-    {
-    if(output || verbose)
+    while((pipe = cmsysProcess_WaitForData(cp, &data, &length, 0)) > 0)
       {
-      // Translate NULL characters in the output into valid text.
-      // Visual Studio 7 puts these characters in the output of its
-      // build process.
-      for(int i=0; i < length; ++i)
+      if(output || outputflag != OUTPUT_NONE)
         {
-        if(data[i] == '\0')
+        // Translate NULL characters in the output into valid text.
+        // Visual Studio 7 puts these characters in the output of its
+        // build process.
+        for(int i=0; i < length; ++i)
           {
-          data[i] = ' ';
+          if(data[i] == '\0')
+            {
+            data[i] = ' ';
+            }
+          }
+        }
+      if ( output )
+        {
+        tempOutput.insert(tempOutput.end(), data, data+length);
+        }
+      if(outputflag != OUTPUT_NONE)
+        {
+        if(outputflag == OUTPUT_MERGE)
+          {
+          cmSystemTools::Stdout(data, length);
+          }
+        else
+          {
+          if(pipe == cmsysProcess_Pipe_STDERR)
+            {
+            cmSystemTools::Stderr(data, length);
+            }
+          else if(pipe == cmsysProcess_Pipe_STDOUT)
+            {
+            cmSystemTools::Stdout(data, length);
+            }
           }
         }
       }
-    if ( output )
-      {
-      tempOutput.insert(tempOutput.end(), data, data+length);
-      }
-    if(verbose)
-      {
-      cmSystemTools::Stdout(data, length);
-      }
     }
-    }
-  
+
   cmsysProcess_WaitForExit(cp, 0);
   if ( output && tempOutput.begin() != tempOutput.end())
     {
     output->append(&*tempOutput.begin(), tempOutput.size());
     }
-  
+
   bool result = true;
   if(cmsysProcess_GetState(cp) == cmsysProcess_State_Exited)
     {
@@ -636,7 +709,7 @@ bool cmSystemTools::RunSingleCommand(std::vector<cmStdString>const& command,
   else if(cmsysProcess_GetState(cp) == cmsysProcess_State_Exception)
     {
     const char* exception_str = cmsysProcess_GetExceptionString(cp);
-    if ( verbose )
+    if ( outputflag != OUTPUT_NONE )
       {
       std::cerr << exception_str << std::endl;
       }
@@ -649,7 +722,7 @@ bool cmSystemTools::RunSingleCommand(std::vector<cmStdString>const& command,
   else if(cmsysProcess_GetState(cp) == cmsysProcess_State_Error)
     {
     const char* error_str = cmsysProcess_GetErrorString(cp);
-    if ( verbose )
+    if ( outputflag != OUTPUT_NONE )
       {
       std::cerr << error_str << std::endl;
       }
@@ -662,7 +735,7 @@ bool cmSystemTools::RunSingleCommand(std::vector<cmStdString>const& command,
   else if(cmsysProcess_GetState(cp) == cmsysProcess_State_Expired)
     {
     const char* error_str = "Process terminated due to timeout\n";
-    if ( verbose )
+    if ( outputflag != OUTPUT_NONE )
       {
       std::cerr << error_str << std::endl;
       }
@@ -672,22 +745,22 @@ bool cmSystemTools::RunSingleCommand(std::vector<cmStdString>const& command,
       }
     result = false;
     }
-  
+
   cmsysProcess_Delete(cp);
   return result;
 }
 
 bool cmSystemTools::RunSingleCommand(
-  const char* command, 
+  const char* command,
   std::string* output,
-  int *retVal, 
+  int *retVal,
   const char* dir,
-  bool verbose,
+  OutputOption outputflag,
   double timeout)
 {
   if(s_DisableRunCommandOutput)
     {
-    verbose = false;
+    outputflag = OUTPUT_NONE;
     }
 
   std::vector<cmStdString> args = cmSystemTools::ParseArguments(command);
@@ -696,17 +769,17 @@ bool cmSystemTools::RunSingleCommand(
     {
     return false;
     }
-  return cmSystemTools::RunSingleCommand(args, output,retVal, 
-                                         dir, verbose, timeout);
+  return cmSystemTools::RunSingleCommand(args, output,retVal,
+                                         dir, outputflag, timeout);
 }
-bool cmSystemTools::RunCommand(const char* command, 
+bool cmSystemTools::RunCommand(const char* command,
                                std::string& output,
                                const char* dir,
                                bool verbose,
                                int timeout)
 {
   int dummy;
-  return cmSystemTools::RunCommand(command, output, dummy, 
+  return cmSystemTools::RunCommand(command, output, dummy,
                                    dir, verbose, timeout);
 }
 
@@ -721,11 +794,11 @@ bool RunCommandViaWin32(const char* command,
                         int timeout)
 {
 #if defined(__BORLANDC__)
-  return 
+  return
     cmWin32ProcessExecution::
-    BorlandRunCommand(command, dir, output, 
-                      retVal, 
-                      verbose, timeout, 
+    BorlandRunCommand(command, dir, output,
+                      retVal,
+                      verbose, timeout,
                       cmSystemTools::GetRunCommandHideConsole());
 #else // Visual studio
   ::SetLastError(ERROR_SUCCESS);
@@ -739,7 +812,7 @@ bool RunCommandViaWin32(const char* command,
     {
     resProc.SetHideWindows(true);
     }
-  
+
   if ( cmSystemTools::GetWindows9xComspecSubstitute() )
     {
     resProc.SetConsoleSpawn(cmSystemTools::GetWindows9xComspecSubstitute() );
@@ -766,7 +839,7 @@ bool RunCommandViaSystem(const char* command,
                          std::string& output,
                          int& retVal,
                          bool verbose)
-{  
+{
   std::cout << "@@ " << command << std::endl;
 
   std::string commandInDir;
@@ -966,9 +1039,9 @@ bool RunCommandViaPopen(const char* command,
 
 // run a command unix uses popen (easy)
 // windows uses system and ShortPath
-bool cmSystemTools::RunCommand(const char* command, 
+bool cmSystemTools::RunCommand(const char* command,
                                std::string& output,
-                               int &retVal, 
+                               int &retVal,
                                const char* dir,
                                bool verbose,
                                int timeout)
@@ -977,7 +1050,7 @@ bool cmSystemTools::RunCommand(const char* command,
     {
     verbose = false;
     }
-  
+
 #if defined(WIN32) && !defined(__CYGWIN__)
   // if the command does not start with a quote, then
   // try to find the program, and if the program can not be
@@ -996,9 +1069,9 @@ bool cmSystemTools::RunCommand(const char* command,
           {
           break;
           }
-        }      
+        }
       }
-    // if there are more than two double quotes use 
+    // if there are more than two double quotes use
     // GetShortPathName, the cmd.exe program in windows which
     // is used by system fails to execute if there are more than
     // one set of quotes in the arguments
@@ -1022,16 +1095,16 @@ bool cmSystemTools::RunCommand(const char* command,
         shortCmd += " ";
         shortCmd += args;
 
-        //return RunCommandViaSystem(shortCmd.c_str(), dir, 
+        //return RunCommandViaSystem(shortCmd.c_str(), dir,
         //                           output, retVal, verbose);
-        //return WindowsRunCommand(shortCmd.c_str(), dir, 
+        //return WindowsRunCommand(shortCmd.c_str(), dir,
         //output, retVal, verbose);
-        return RunCommandViaWin32(shortCmd.c_str(), dir, 
+        return RunCommandViaWin32(shortCmd.c_str(), dir,
                                   output, retVal, verbose, timeout);
         }
       else
         {
-        cmSystemTools::Error("Could not parse command line with quotes ", 
+        cmSystemTools::Error("Could not parse command line with quotes ",
                              command);
         }
       }
@@ -1095,7 +1168,7 @@ bool cmSystemTools::cmCopyFile(const char* source, const char* destination)
   return Superclass::CopyFileAlways(source, destination);
 }
 
-bool cmSystemTools::CopyFileIfDifferent(const char* source, 
+bool cmSystemTools::CopyFileIfDifferent(const char* source,
   const char* destination)
 {
   return Superclass::CopyFileIfDifferent(source, destination);
@@ -1154,48 +1227,10 @@ bool cmSystemTools::RenameFile(const char* oldname, const char* newname)
 bool cmSystemTools::ComputeFileMD5(const char* source, char* md5out)
 {
 #if defined(CMAKE_BUILD_WITH_CMAKE)
-  if(!cmSystemTools::FileExists(source))
-    {
-    return false;
-    }
-
-  // Open files
-#if defined(_WIN32) || defined(__CYGWIN__)
-  cmsys_ios::ifstream fin(source, cmsys_ios::ios::binary | cmsys_ios::ios::in);
-#else
-  cmsys_ios::ifstream fin(source);
-#endif
-  if(!fin)
-    {
-    return false;
-    }
-
-  cmsysMD5* md5 = cmsysMD5_New();
-  cmsysMD5_Initialize(md5);
-
-  // Should be efficient enough on most system:
-  const int bufferSize = 4096;
-  char buffer[bufferSize];
-  unsigned char const* buffer_uc =
-    reinterpret_cast<unsigned char const*>(buffer);
-  // This copy loop is very sensitive on certain platforms with
-  // slightly broken stream libraries (like HPUX).  Normally, it is
-  // incorrect to not check the error condition on the fin.read()
-  // before using the data, but the fin.gcount() will be zero if an
-  // error occurred.  Therefore, the loop should be safe everywhere.
-  while(fin)
-    {
-    fin.read(buffer, bufferSize);
-    if(int gcount = static_cast<int>(fin.gcount()))
-      {
-      cmsysMD5_Append(md5, buffer_uc, gcount);
-      }
-    }
-  cmsysMD5_FinalizeHex(md5, md5out);
-  cmsysMD5_Delete(md5);
-
-  fin.close();
-  return true;
+  cmCryptoHashMD5 md5;
+  std::string str = md5.HashFile(source);
+  strncpy(md5out, str.c_str(), 32);
+  return !str.empty();
 #else
   (void)source;
   (void)md5out;
@@ -1207,13 +1242,8 @@ bool cmSystemTools::ComputeFileMD5(const char* source, char* md5out)
 std::string cmSystemTools::ComputeStringMD5(const char* input)
 {
 #if defined(CMAKE_BUILD_WITH_CMAKE)
-  char md5out[32];
-  cmsysMD5* md5 = cmsysMD5_New();
-  cmsysMD5_Initialize(md5);
-  cmsysMD5_Append(md5, reinterpret_cast<unsigned char const*>(input), -1);
-  cmsysMD5_FinalizeHex(md5, md5out);
-  cmsysMD5_Delete(md5);
-  return std::string(md5out, 32);
+  cmCryptoHashMD5 md5;
+  return md5.HashString(input);
 #else
   (void)input;
   cmSystemTools::Message("md5sum not supported in bootstrapping mode","Error");
@@ -1226,7 +1256,7 @@ void cmSystemTools::Glob(const char *directory, const char *regexp,
 {
   cmsys::Directory d;
   cmsys::RegularExpression reg(regexp);
-  
+
   if (d.Load(directory))
     {
     size_t numf;
@@ -1279,7 +1309,7 @@ void cmSystemTools::GlobDirs(const char *fullPath,
 }
 
 
-void cmSystemTools::ExpandList(std::vector<std::string> const& arguments, 
+void cmSystemTools::ExpandList(std::vector<std::string> const& arguments,
                                std::vector<std::string>& newargs)
 {
   std::vector<std::string>::const_iterator i;
@@ -1378,8 +1408,8 @@ void cmSystemTools::ExpandListArgument(const std::string& arg,
     }
 }
 
-bool cmSystemTools::SimpleGlob(const cmStdString& glob, 
-                               std::vector<cmStdString>& files, 
+bool cmSystemTools::SimpleGlob(const cmStdString& glob,
+                               std::vector<cmStdString>& files,
                                int type /* = 0 */)
 {
   files.clear();
@@ -1419,8 +1449,8 @@ bool cmSystemTools::SimpleGlob(const cmStdString& glob,
           {
           continue;
           }
-        if ( sfname.size() >= ppath.size() && 
-             sfname.substr(0, ppath.size()) == 
+        if ( sfname.size() >= ppath.size() &&
+             sfname.substr(0, ppath.size()) ==
              ppath )
           {
           files.push_back(fname);
@@ -1440,10 +1470,10 @@ cmSystemTools::FileFormat cmSystemTools::GetFileFormat(const char* cext)
     }
   //std::string ext = cmSystemTools::LowerCase(cext);
   std::string ext = cext;
-  if ( ext == "c" || ext == ".c" || 
-       ext == "m" || ext == ".m" 
+  if ( ext == "c" || ext == ".c" ||
+       ext == "m" || ext == ".m"
     ) { return cmSystemTools::C_FILE_FORMAT; }
-  if ( 
+  if (
     ext == "C" || ext == ".C" ||
     ext == "M" || ext == ".M" ||
     ext == "c++" || ext == ".c++" ||
@@ -1452,22 +1482,22 @@ cmSystemTools::FileFormat cmSystemTools::GetFileFormat(const char* cext)
     ext == "cxx" || ext == ".cxx" ||
     ext == "mm" || ext == ".mm"
     ) { return cmSystemTools::CXX_FILE_FORMAT; }
-  if ( 
+  if (
     ext == "f" || ext == ".f" ||
     ext == "F" || ext == ".F" ||
     ext == "f77" || ext == ".f77" ||
     ext == "f90" || ext == ".f90" ||
     ext == "for" || ext == ".for" ||
-    ext == "f95" || ext == ".f95" 
+    ext == "f95" || ext == ".f95"
     ) { return cmSystemTools::FORTRAN_FILE_FORMAT; }
   if ( ext == "java" || ext == ".java" )
     { return cmSystemTools::JAVA_FILE_FORMAT; }
-  if ( 
-    ext == "H" || ext == ".H" || 
-    ext == "h" || ext == ".h" || 
+  if (
+    ext == "H" || ext == ".H" ||
+    ext == "h" || ext == ".h" ||
     ext == "h++" || ext == ".h++" ||
-    ext == "hm" || ext == ".hm" || 
-    ext == "hpp" || ext == ".hpp" || 
+    ext == "hm" || ext == ".hm" ||
+    ext == "hpp" || ext == ".hpp" ||
     ext == "hxx" || ext == ".hxx" ||
     ext == "in" || ext == ".in" ||
     ext == "txx" || ext == ".txx"
@@ -1480,18 +1510,18 @@ cmSystemTools::FileFormat cmSystemTools::GetFileFormat(const char* cext)
        ext == "a" || ext == ".a")
     { return cmSystemTools::STATIC_LIBRARY_FILE_FORMAT; }
   if ( ext == "o" || ext == ".o" ||
-       ext == "obj" || ext == ".obj") 
+       ext == "obj" || ext == ".obj")
     { return cmSystemTools::OBJECT_FILE_FORMAT; }
 #ifdef __APPLE__
-  if ( ext == "dylib" || ext == ".dylib" ) 
+  if ( ext == "dylib" || ext == ".dylib" )
     { return cmSystemTools::SHARED_LIBRARY_FILE_FORMAT; }
-  if ( ext == "so" || ext == ".so" || 
-       ext == "bundle" || ext == ".bundle" ) 
-    { return cmSystemTools::MODULE_FILE_FORMAT; } 
+  if ( ext == "so" || ext == ".so" ||
+       ext == "bundle" || ext == ".bundle" )
+    { return cmSystemTools::MODULE_FILE_FORMAT; }
 #else // __APPLE__
-  if ( ext == "so" || ext == ".so" || 
-       ext == "sl" || ext == ".sl" || 
-       ext == "dll" || ext == ".dll" ) 
+  if ( ext == "so" || ext == ".so" ||
+       ext == "sl" || ext == ".sl" ||
+       ext == "dll" || ext == ".dll" )
     { return cmSystemTools::SHARED_LIBRARY_FILE_FORMAT; }
 #endif // __APPLE__
   return cmSystemTools::UNKNOWN_FILE_FORMAT;
@@ -1687,7 +1717,7 @@ bool cmSystemTools::IsPathToFramework(const char* path)
   return false;
 }
 
-bool cmSystemTools::CreateTar(const char* outFileName, 
+bool cmSystemTools::CreateTar(const char* outFileName,
                               const std::vector<cmStdString>& files,
                               bool gzip, bool bzip2, bool verbose)
 {
@@ -1752,7 +1782,7 @@ namespace{
   static time_t          now;
   size_t u_width = 6;
   size_t gs_width = 13;
-  
+
   /*
    * We avoid collecting the entire list in memory at once by
    * listing things as we see them.  However, that also means we can't
@@ -1767,7 +1797,7 @@ namespace{
   fprintf(out, "%s %d ",
           archive_entry_strmode(entry),
           archive_entry_nlink(entry));
-  
+
   /* Use uname if it's present, else uid. */
   p = archive_entry_uname(entry);
   if ((p == NULL) || (*p == '\0'))
@@ -1788,7 +1818,7 @@ namespace{
     {
     fprintf(out, "%s", p);
     w = strlen(p);
-    } 
+    }
   else
     {
     sprintf(tmp, "%lu",
@@ -1796,20 +1826,20 @@ namespace{
     w = strlen(tmp);
     fprintf(out, "%s", tmp);
     }
-  
+
   /*
    * Print device number or file size, right-aligned so as to make
    * total width of group and devnum/filesize fields be gs_width.
    * If gs_width is too small, grow it.
    */
   if (archive_entry_filetype(entry) == AE_IFCHR
-      || archive_entry_filetype(entry) == AE_IFBLK) 
+      || archive_entry_filetype(entry) == AE_IFBLK)
     {
     sprintf(tmp, "%lu,%lu",
             (unsigned long)archive_entry_rdevmajor(entry),
             (unsigned long)archive_entry_rdevminor(entry));
     }
-  else 
+  else
     {
     /*
      * Note the use of platform-dependent macros to format
@@ -1830,7 +1860,7 @@ namespace{
 #define HALF_YEAR (time_t)365 * 86400 / 2
 #if defined(_WIN32) && !defined(__CYGWIN__)
   /* Windows' strftime function does not support %e format. */
-#define DAY_FMT  "%d"  
+#define DAY_FMT  "%d"
 #else
 #define DAY_FMT  "%e"  /* Day number without leading zeros */
 #endif
@@ -1860,15 +1890,19 @@ namespace{
 #ifdef __BORLANDC__
 # pragma warn -8066 /* unreachable code */
 #endif
-  
+
 long copy_data(struct archive *ar, struct archive *aw)
 {
   long r;
   const void *buff;
   size_t size;
+#if defined(ARCHIVE_VERSION_NUMBER) && ARCHIVE_VERSION_NUMBER >= 3000000
+  __LA_INT64_T offset;
+#else
   off_t offset;
-  
-  for (;;) 
+#endif
+
+  for (;;)
     {
     r = archive_read_data_block(ar, &buff, &size, &offset);
     if (r == ARCHIVE_EOF)
@@ -1880,7 +1914,7 @@ long copy_data(struct archive *ar, struct archive *aw)
       return (r);
       }
     r = archive_write_data_block(aw, buff, size, offset);
-    if (r != ARCHIVE_OK) 
+    if (r != ARCHIVE_OK)
       {
       cmSystemTools::Message("archive_write_data_block()",
                              archive_error_string(aw));
@@ -1890,7 +1924,7 @@ long copy_data(struct archive *ar, struct archive *aw)
   return r;
 }
 
-bool extract_tar(const char* outFileName, bool verbose, 
+bool extract_tar(const char* outFileName, bool verbose,
                  bool extract)
 {
   struct archive* a = archive_read_new();
@@ -1900,12 +1934,12 @@ bool extract_tar(const char* outFileName, bool verbose,
   struct archive_entry *entry;
   int r = archive_read_open_file(a, outFileName, 10240);
   if(r)
-    { 
+    {
     cmSystemTools::Error("Problem with archive_read_open_file(): ",
                          archive_error_string(a));
     return false;
     }
-  for (;;) 
+  for (;;)
     {
     r = archive_read_next_header(a, &entry);
     if (r == ARCHIVE_EOF)
@@ -1948,7 +1982,7 @@ bool extract_tar(const char* outFileName, bool verbose,
         cmSystemTools::Error("Current file:",
                              archive_entry_pathname(entry));
         }
-      else 
+      else
         {
         copy_data(a, ext);
         r = archive_write_finish_entry(ext);
@@ -1970,9 +2004,9 @@ bool extract_tar(const char* outFileName, bool verbose,
 
 }
 }
-#endif 
+#endif
 
-bool cmSystemTools::ExtractTar(const char* outFileName, 
+bool cmSystemTools::ExtractTar(const char* outFileName,
                                bool , bool verbose)
 {
 #if defined(CMAKE_BUILD_WITH_CMAKE)
@@ -1984,7 +2018,7 @@ bool cmSystemTools::ExtractTar(const char* outFileName,
 #endif
 }
 
-bool cmSystemTools::ListTar(const char* outFileName, 
+bool cmSystemTools::ListTar(const char* outFileName,
                             bool ,
                             bool verbose)
 {
@@ -2100,8 +2134,8 @@ int cmSystemTools::WaitForLine(cmsysProcess* process, std::string& line,
 }
 
 void cmSystemTools::DoNotInheritStdPipes()
-{   
-#ifdef _WIN32  
+{
+#ifdef _WIN32
   // Check to see if we are attached to a console
   // if so, then do not stop the inherited pipes
   // or stdout and stderr will not show up in dos
@@ -2232,6 +2266,71 @@ bool cmSystemTools::FileTimeSet(const char* fname, cmSystemToolsFileTime* t)
   return true;
 }
 
+//----------------------------------------------------------------------------
+#ifdef _WIN32
+# ifndef CRYPT_SILENT
+#  define CRYPT_SILENT 0x40 /* Not defined by VS 6 version of header.  */
+# endif
+static int WinCryptRandom(void* data, size_t size)
+{
+  int result = 0;
+  HCRYPTPROV hProvider = 0;
+  if(CryptAcquireContextW(&hProvider, 0, 0, PROV_RSA_FULL,
+                          CRYPT_VERIFYCONTEXT | CRYPT_SILENT))
+    {
+    result = CryptGenRandom(hProvider, (DWORD)size, (BYTE*)data)? 1:0;
+    CryptReleaseContext(hProvider, 0);
+    }
+  return result;
+}
+#endif
+
+//----------------------------------------------------------------------------
+unsigned int cmSystemTools::RandomSeed()
+{
+#if defined(_WIN32) && !defined(__CYGWIN__)
+  unsigned int seed = 0;
+
+  // Try using a real random source.
+  if(WinCryptRandom(&seed, sizeof(seed)))
+    {
+    return seed;
+    }
+
+  // Fall back to the time and pid.
+  FILETIME ft;
+  GetSystemTimeAsFileTime(&ft);
+  unsigned int t1 = static_cast<unsigned int>(ft.dwHighDateTime);
+  unsigned int t2 = static_cast<unsigned int>(ft.dwLowDateTime);
+  unsigned int pid = static_cast<unsigned int>(GetCurrentProcessId());
+  return t1 ^ t2 ^ pid;
+#else
+  union
+  {
+    unsigned int integer;
+    char bytes[sizeof(unsigned int)];
+  } seed;
+
+  // Try using a real random source.
+  std::ifstream fin("/dev/urandom");
+  if(fin && fin.read(seed.bytes, sizeof(seed)) &&
+     fin.gcount() == sizeof(seed))
+    {
+    return seed.integer;
+    }
+
+  // Fall back to the time and pid.
+  struct timeval t;
+  gettimeofday(&t, 0);
+  unsigned int pid = static_cast<unsigned int>(getpid());
+  unsigned int tv_sec = static_cast<unsigned int>(t.tv_sec);
+  unsigned int tv_usec = static_cast<unsigned int>(t.tv_usec);
+  // Since tv_usec never fills more than 11 bits we shift it to fill
+  // in the slow-changing high-order bits of tv_sec.
+  return tv_sec ^ (tv_usec << 21) ^ pid;
+#endif
+}
+
 //----------------------------------------------------------------------------
 static std::string cmSystemToolsExecutableDirectory;
 void cmSystemTools::FindExecutableDirectory(const char* argv0)
@@ -2603,7 +2702,7 @@ bool cmSystemTools::VersionCompare(cmSystemTools::CompareOp op,
     else if(lhs[i] > rhs[i])
       {
       // lhs > rhs, so true if operation is GREATER
-        return op == cmSystemTools::OP_GREATER;
+      return op == cmSystemTools::OP_GREATER;
       }
     }
   // lhs == rhs, so true if operation is EQUAL
@@ -2833,3 +2932,35 @@ bool cmSystemTools::RepeatedRemoveDirectory(const char* dir)
     }
   return false;
 }
+
+//----------------------------------------------------------------------------
+std::vector<std::string> cmSystemTools::tokenize(const std::string& str,
+                                  const std::string& sep)
+{
+  std::vector<std::string> tokens;
+  std::string::size_type tokend = 0;
+
+  do
+    {
+    std::string::size_type tokstart=str.find_first_not_of(sep, tokend);
+    if (tokstart==std::string::npos)
+      {
+      break;    // no more tokens
+      }
+    tokend=str.find_first_of(sep,tokstart);
+    if (tokend==std::string::npos)
+      {
+      tokens.push_back(str.substr(tokstart));
+      }
+    else
+      {
+      tokens.push_back(str.substr(tokstart,tokend-tokstart));
+      }
+    } while (tokend!=std::string::npos);
+
+  if (tokens.empty())
+    {
+    tokens.push_back("");
+    }
+  return tokens;
+}
